# - *- coding: utf- 8 - *-

from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Screens.InfoBar import InfoBar, MoviePlayer
from Screens.MovieSelection import MovieSelection, MovieContextMenu
from Screens.TimerEntry import TimerEntry
from Screens.MessageBox import MessageBox
from Components.Label import Label
from Components.ScrollLabel import ScrollLabel
from Components.ProgressBar import ProgressBar
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap, HelpableActionMap
from Components.AVSwitch import AVSwitch
from Components.Pixmap import Pixmap, MultiPixmap
from Components.config import config, fileExists
from Components.TimerSanityCheck import TimerSanityCheck
from enigma import gPixmapPtr, ePicLoad, iServiceInformation, eServiceReference, eEPGCache, eTimer, eServiceCenter, ePoint, eSize
from RecordTimer import RecordTimerEntry, parseEvent
from ServiceReference import ServiceReference
from thread import start_new_thread
from skin import parseColor

from .TVSViewerConfig import TVSViewerConfigScreen, pluginPath, skinPath, tvsRefFileIntern, tvsRefFileAEL, writeLog, isAELInstalled, isTMDbInstalled, coverPath
from .TVSViewerTVSSetup import TVSViewer_TVSSetup

import time
from datetime import datetime,timedelta
import json
import requests
import shutil
import skin


if isTMDbInstalled:
	from Plugins.Extensions.tmdb.tmdb import tmdbScreen
if isAELInstalled:
	from Plugins.Extensions.AdvancedEventLibrary.AdvancedEventLibrarySystem import TVSSetup as aelTVSSetup

#######################################################
class TVSViewer(Screen, HelpableScreen):
	skin.loadSkin(skinPath + "TVSViewer.xml")
	ALLOW_SUSPEND = True
	def __init__(self, session, callEvent=None, callServiceRef=None, callType=None):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		self.session = session
		self.title = "TV-Spielfilm-Viewer"
		self.skinName = "TVSViewer"
		
		#init
		self.callEvent = callEvent
		self.callServiceRef = callServiceRef
		self.callType = callType #EPG/MovieList
		self.callServiceRefInBouquet = False
		
		self.sTVSCode = ""
		self.sTVSResultList = []
		self.sTVSSearchedDatesList = []
		self.sTVSAgeFilterOff = False
		self.serviceList = []
		self.liveService = None
		self.currentService = None
		self.currentRefStr = ""
		self.currentServiceName = ""
		self.currentEntry = None
		self.currentAirtime = 0
		self.nextPrevAirtime = 0
		self.primeTime = 0
		self.primeTimeMode = 0
		
		self.coverStartup = True
		self.coverDelayStartup = config.plugins.TVSViewer.coverDelayStartup.value
		self.coverDelayZap = config.plugins.TVSViewer.coverDelayZap.value
		self.showCoverDelayTimer = eTimer()
		self.showCoverDelayTimer.callback.append(self.onCoverDelayTimer)
		self.hideEndMarkerTimer = eTimer()
		self.hideEndMarkerTimer.callback.append(self.onHideEndMarkerTimer)
		
		self.movieSelection = None
		self.movieContextMenu = None
		self.movieList = []
		
		self["title"] = StaticText(self.title)
		self["background"] = Label()
		self["chRef"] = StaticText()
		self["chName"] = Label()
		self["eventtitle"] = Label()
		self["infoline"] = Label()
		self["eventtime"] = Label()
		self["eventdate"] = Label()
		self["eventtimebig"] = Label()
		self["eventdatebig"] = Label()
		self["duration"] = Label()
		self["description"] = ScrollLabel()
		self["casting"] = ScrollLabel()
		
		self["primeTimeMode"] = Pixmap()
		self["progressBar"] = ProgressBar()
		
		self["ratingAction"] = MultiPixmap()
		self["ratingDemanding"] = MultiPixmap()
		self["ratingErotic"] = MultiPixmap()
		self["ratingHumor"] = MultiPixmap()
		self["ratingSuspense"] = MultiPixmap()
		
		self["isNew"] = MultiPixmap()
		self["isTipOfTheDay"] = MultiPixmap()
		self["isTopTip"] = MultiPixmap()
		
		self["cover"] = Pixmap()
		self["coverBig"] = Pixmap()
		self["coverBigBlur"] = Pixmap()
		self["trailer"] = Pixmap()
		
		self["endLeftMarker"] = Pixmap()
		self["endRightMarker"] = Pixmap()
		
		self["key_red"] = Label(_("PrimeTime"))
		self["key_green"] = Label(_("Aktuelle Uhrzeit"))
		self["key_yellow"] = Label()
		self["key_blue"] = Label()
		
		myActions = {
			"key_red": (self.keyRed, _("Zur PrimeTime springen / Play")),
			"key_green": (self.keyGreen, _("Zur aktuellen Uhrzeit springen")),
			"key_blue": (self.keyBlue, _("TVS-Setup aufrufen")),
			"key_cancel": (self.keyCancel, _("Plugin beenden")),
			"key_up": (self.keyUp, _("Beschreibung nach oben scrollen")),
			"key_down": (self.keyDown, _("Beschreibung nach unten scrollen")),
			"key_channel_up": (self.keyChannelUp, _("Sender / Film +")),
			"key_channel_down": (self.keyChannelDown, _("Sender / Film -")),
			"key_rewind": (self.keyRewind, _("Besetzungsliste nach oben scrollen")),
			"key_fastforward": (self.keyFastForward, _("Besetzungsliste nach unten scrollen")),
			"key_left": (self.keyLeft, _("Vorherige Sendung anzeigen")),
			"key_right": (self.keyRight, _("Nächste Sendung anzeigen")),
			"key_ok": (self.keyOK, _("Zap & Exit")),
			"key_previous": (self.keyPrevious, _("- 1Tag springen")),
			"key_next": (self.keyNext, _("+ 1Tag springen")),
			"key_menu": (self.keyMenu, _("Einstellungen anzeigen")),
			"key_info": (self.keyInfo, _("Startsender anzeigen")),
			"key_pvr": (self.keyPVR, _("Cover-Big einblenden/ausblenden")),
			"key_play": (self.keyPlay, _("Trailer abspielen")),
			"key_rec": (self.keyRec, _("Timereintrag erstellen")),
		}
		
		if isTMDbInstalled:
			self["key_yellow"].setText(_("TMDb"))
			myActions.update({"key_yellow": (self.keyYellow, _("TMDb-Plugin aufrufen"))})
		
		if isAELInstalled and config.plugins.TVSViewer.tvsSetup.value == "AEL":
			self["key_blue"].setText(_("TVS-Setup (AEL)"))
			self.tvsRefFile = tvsRefFileAEL
			self.tvsSetupAEL = True
		else:
			self["key_blue"].setText(_("TVS-Setup"))
			self.tvsRefFile = tvsRefFileIntern
			self.tvsSetupAEL = False
			
		self["myActionMap"] = HelpableActionMap(self, "TVSViewerActions",myActions,-1)
		
		self.onLayoutFinish.append(self.startup)
		self.onClose.append(self.updateMovieSelection)
		self.onClose.append(self.cleanUp)
		
	def startup(self):
		writeLog("#### startup ####")
		
		#background
		bkColor = config.plugins.TVSViewer.skin.value
		self["background"].instance.setBackgroundColor(parseColor(bkColor))
		
		self["primeTimeMode"].hide()
		self["endLeftMarker"].hide()
		self["endRightMarker"].hide()
		self["progressBar"].hide()
		self["trailer"].hide()
		self["cover"].hide()
		self["coverBig"].hide()
		self["coverBigBlur"].hide()
		
		self.primeTime = self.getPrimeTime()
		
		#Prepare ChannelInit
		if self.callType == "EPG":
			writeLog("callType: EPG")
			if self.callServiceRef:
				self.serviceList = self.getServiceList()[0]
				#Check if callServiceRef in Bouquet
				for service in self.serviceList:
					if str(service) == str(self.callServiceRef):
						self.callServiceRefInBouquet = True
						break
			self.channelInit(self.callServiceRef)
		elif self.callType == "MovieList":
			writeLog("callType: MovieList")
			self["key_red"].setText("")
			self["key_green"].setText("")
			for screen,shown in self.session.dialog_stack:
				if isinstance(screen, MovieSelection):
					self.movieSelection = screen
					self["key_red"].setText("Play")
				if isinstance(screen, MovieContextMenu):
					self.movieContextMenu = screen
			self.channelInit(self.callServiceRef)
		else:
			#MQB: Live or MoviePlayer
			playingService = self.getPlayingService()
			isMoviePlayer = False
			for screen,shown in self.session.dialog_stack:
				if isinstance(screen, MoviePlayer):
					isMoviePlayer = True
					break
			if isMoviePlayer:
				self.callType = "MoviePlayer"
				writeLog("callType: MoviePlayer")
				self["key_red"].setText("")
				self["key_green"].setText("")
				self["key_blue"].setText("")
			else:
				self.callType = "Live"
				writeLog("callType: Live")
				self.serviceList = self.getServiceList()[0]
				#Check if playingService in Bouquet
				for service in self.serviceList:
					if str(service) == str(playingService):
						self.liveService = playingService
						break
			
			self.channelInit(playingService)
			
	#============== channelInit ======================
	def channelInit(self, service):
		writeLog("---- ChannelInit ----")
		self.currentService = service
		
		self.sTVSCode = ""
		self.sTVSResultList = []
		self.sTVSSearchedDatesList = []
		self.currentEntry = None
		self.currentAirtime = 0
		self.nextPrevAirtime = 0
		
		info = service and service.info()
		
		#currentRefStr
		self.currentRefStr = ""
		if self.callType == "Live" or self.callType == "EPG":
			if service:
				self.currentRefStr = str(service)
		elif info:
			self.currentRefStr = info.getInfoString(service.ref, iServiceInformation.sServiceref)
		
		if service:
			self.currentServiceName = str(service.getServiceName())
		
		self["chRef"].setText(self.currentRefStr)
		self["chName"].setText(self.currentServiceName)
		self.sTVSCode = self.getTVSCode(self.currentRefStr)
		
		writeLog("ServiceName: " + self.currentServiceName)
		writeLog("REF: " + str(self.currentRefStr))
		writeLog("TVS-Code: " + str(self.sTVSCode))
		
		#sAirtime
		sAirtime = 0
		if self.callType == "EPG" and self.callEvent:
			eventStart = self.callEvent.getBeginTime()
			eventEnd = eventStart + self.callEvent.getDuration()
			sAirtime = eventStart + ((eventEnd - eventStart) / 4)
			self.sTVSAgeFilterOff = True
		elif self.callType != "Live" and (info and info.getInfo(service.ref, iServiceInformation.sTimeCreate) > 0):
			serviceStart = info.getInfo(service.ref, iServiceInformation.sTimeCreate)
			serviceDuration = info.getLength(service.ref)
			sAirtime = serviceStart + (serviceDuration / 4)
			self.sTVSAgeFilterOff = True	
		elif self.primeTimeMode:
			sAirtime = self.primeTimeMode
		if sAirtime <= 0:
			sAirtime = int(time.time())
		writeLog("sAirtime: " + str(sAirtime))
		self.searchByAirtime(sAirtime)
	
	
	#============== Key-Actions ======================	
	def keyRed(self):
		#Primetime
		writeLog("keyRed")
		if self.hideCoverBigBefore(): return
		if self.callType == "Live" or self.callType == "EPG":
			self.nextPrevAirtime = 0
			self.setPrimeTimeMode(self.primeTime)
			self.searchByAirtime(self.primeTime)
		#PlayMovie
		elif self.callType == "MovieList" and self.movieSelection and self.currentService.getType() != 2:
			if self.movieContextMenu:
				self.movieContextMenu.close()
			self.updateMovieSelection()
			self.movieSelection.movieSelected()
			self.close()
		
	def keyGreen(self):
		#Aktuelle Uhrzeit
		writeLog("keyGreen")
		if self.hideCoverBigBefore(): return
		if self.callType == "Live" or self.callType == "EPG":
			self.nextPrevAirtime = 0
			self.setPrimeTimeMode(0)
			self.searchByAirtime(int(time.time()))
		
	def keyYellow(self):
		#TMDb
		writeLog("keyYellow")
		if self.hideCoverBigBefore(): return
		if isTMDbInstalled:
			if self.currentEntry:
				sTitle = self.currentEntry["eventtitle"]
			else:
				sTitle = ""
			self.session.open(tmdbScreen, sTitle, 0)
			
	def keyBlue(self):
		#TVS-Setup
		writeLog("keyBlue")
		if self.hideCoverBigBefore(): return
		if self.tvsSetupAEL:
			self.session.openWithCallback(self.cbTVSSetup, aelTVSSetup)
		else:
			if self.callType == "Live":
				self.session.openWithCallback(self.cbTVSSetup, TVSViewer_TVSSetup, self.getServiceList(), self.currentRefStr, self.currentServiceName)
			elif self.callType == "EPG":
				if self.callServiceRefInBouquet:
					self.session.openWithCallback(self.cbTVSSetup, TVSViewer_TVSSetup, self.getServiceList(), self.currentRefStr, self.currentServiceName)
				else:
					self.session.openWithCallback(self.cbTVSSetup, TVSViewer_TVSSetup, None, self.currentRefStr,self.currentServiceName)
			elif self.callType == "MovieList":
				self.session.openWithCallback(self.cbTVSSetup, TVSViewer_TVSSetup, None, self.currentRefStr,self.currentRefStr)
	
	def cbTVSSetup(self):
		if self.currentService:
			self.channelInit(self.currentService)
		
	def keyOK(self):
		#Zap & Exit
		writeLog("keyOK")
		if self.hideCoverBigBefore(): return
		#if not self.liveService: return
		try:
			if self.currentService and self.liveService and self.currentService.ref != self.liveService.ref:
				if InfoBar and InfoBar.instance:
					servicelist = InfoBar.instance.servicelist
					servicelist.setCurrentSelection(self.currentService.ref)
					servicelist.zap()
		except Exception as ex:
			pass
		self.close()
		
	def keyCancel(self):
		#Close
		writeLog("keyCancel")
		if self.hideCoverBigBefore(): return
		self.close()
		
	def keyUp(self):
		#Description up
		writeLog("keyUp")
		if self.hideCoverBigBefore(): return
		self["description"].pageUp()
			
	def keyDown(self):
		#Description down
		writeLog("keyDown")
		if self.hideCoverBigBefore(): return
		self["description"].pageDown()
			
	def keyChannelUp(self):
		#Channel up
		writeLog("keyChannelUp")
		if self.hideCoverBigBefore(): return		
		#NextMovie
		if self.callType == "MovieList" and self.movieSelection:
			newMovieService = self.getNextPrevMovieRef(1)
			if newMovieService:
				self.channelInit(ServiceReference(newMovieService))
		#NextChannel
		elif self.callType == "Live" or (self.callType == "EPG" and self.callServiceRefInBouquet):
			newService = self.getNextPrevService(1)
			if newService:
				self.channelInit(newService)
			
	def keyChannelDown(self):
		#Channel down
		writeLog("keyChannelDown")
		if self.hideCoverBigBefore(): return
		#PrevMovie
		if self.callType == "MovieList" and self.movieSelection:
			newMovieService = self.getNextPrevMovieRef(-1)
			if newMovieService:
				self.channelInit(ServiceReference(newMovieService))
		#PrevChannel
		elif self.callType == "Live" or (self.callType == "EPG" and self.callServiceRefInBouquet):
			newService = self.getNextPrevService(-1)
			if newService:
				self.channelInit(newService)
		
	def keyRewind(self):
		#Castinglist up
		writeLog("keyRewind")
		if self.hideCoverBigBefore(): return
		self["casting"].pageUp()
			
	def keyFastForward(self):
		#Castinglist down
		writeLog("keyFastForward")
		if self.hideCoverBigBefore(): return
		self["casting"].pageDown()
		
	def keyLeft(self):
		writeLog("keyLeft")
		if self.hideCoverBigBefore(): return
		if not self.currentAirtime: return
		self.nextPrevAirtime = 0
		self.setPrimeTimeMode(0)
		prevEntryAvailable = False
		airtimeIndex = self.findAirtimeIndex(self.currentAirtime)
		if airtimeIndex > 0:
			prevEntryAvailable = True
		elif airtimeIndex == 0:
			#loadPrevDay
			sTVSDate = self.tvsDateAdd(self.sTVSResultList[0]["tvsDate"],-1) 
			self.getTVSpielfilm(self.sTVSCode,sTVSDate)
			airtimeIndex = self.findAirtimeIndex(self.currentAirtime)
			if airtimeIndex > 0:
				prevEntryAvailable = True
		
		if prevEntryAvailable:
			entry = self.sTVSResultList[airtimeIndex-1]
			self.showEntry(entry)
		else:
			self.showEndMarker("left")
			
	def keyRight(self):
		writeLog("keyRight")
		if self.hideCoverBigBefore(): return
		if not self.currentAirtime: return
		self.nextPrevAirtime = 0
		self.setPrimeTimeMode(0)
		nextEntryAvailable = False
		airtimeIndex = self.findAirtimeIndex(self.currentAirtime)
		if airtimeIndex >= 0:
			if airtimeIndex < len(self.sTVSResultList)-1:
				nextEntryAvailable = True
			elif airtimeIndex == len(self.sTVSResultList)-1:
				#loadNextDay
				sTVSDate = self.tvsDateAdd(self.sTVSResultList[-1]["tvsDate"],1)
				
				self.getTVSpielfilm(self.sTVSCode,sTVSDate)
				airtimeIndex = self.findAirtimeIndex(self.currentAirtime)
				if airtimeIndex >= 0 and airtimeIndex < len(self.sTVSResultList)-1:
					nextEntryAvailable = True
		if nextEntryAvailable:
			entry = self.sTVSResultList[airtimeIndex+1]
			self.showEntry(entry)
		else:
			self.showEndMarker("right")
				
	def keyPrevious(self):
		writeLog("keyPrevious")
		if self.hideCoverBigBefore(): return
		if (self.callType == "Live" or self.callType == "EPG") and self.currentAirtime:
			self.setPrimeTimeMode(0)
			if not self.nextPrevAirtime:
				self.nextPrevAirtime = self.currentAirtime
			sAirtime = self.nextPrevAirtime - 86400
			self.searchByAirtime(sAirtime)
		
	def keyNext(self):
		writeLog("keyNext")
		if self.hideCoverBigBefore(): return
		if (self.callType == "Live" or self.callType == "EPG") and self.currentAirtime:
			self.setPrimeTimeMode(0)
			if not self.nextPrevAirtime:
				self.nextPrevAirtime = self.currentAirtime
			sAirtime = self.nextPrevAirtime + 86400
			self.searchByAirtime(sAirtime)
		
	def keyMenu(self):
		writeLog("keyMenu")
		if self.hideCoverBigBefore(): return
		self.session.open(TVSViewerConfigScreen)
	
	def keyInfo(self):
		writeLog("keyInfo")
		if self.hideCoverBigBefore(): return
		if self.liveService:
			self.channelInit(self.liveService)
		elif self.callServiceRef:
			self.channelInit(self.callServiceRef)
	
	def keyPVR(self):
		writeLog("keyPVR")
		if self.hideCoverBigBefore(): return
		if self["cover"].visible:
			self.showCover(self["coverBig"],coverPath)
			self["coverBigBlur"].show()
		
	def keyPlay(self):
		writeLog("keyPlay")
		if self.hideCoverBigBefore(): return
		try:
			if not self.currentEntry: return
			url = self.currentEntry["trailerUrl"]
			if url.endswith("mp4"):
				ref = eServiceReference(4097, 0, url)
				ref.setName(self.currentEntry["eventtitle"])
				self.session.open(TVSViewer_MoviePlayer, ref)
		except Exception as ex:
			pass
			
	def keyRec(self):
		writeLog("keyRec")
		if self.hideCoverBigBefore(): return
		if self.callType == "Live" and self.currentEntry and self.currentService:
			self.session.openWithCallback(self.cbTimerEPG, TVSViewer_TimerEPG, self.currentEntry, self.currentService)
		
	def cbTimerEPG(self, timer):
		if timer:
			self.session.openWithCallback(self.cbTimerAdd, TimerEntry, timer)
			
	def cbTimerAdd(self, answer, instantTimer = False):
		if answer[0]:
			entry = answer[1]
			simulTimerList = self.session.nav.RecordTimer.record(entry)
			if simulTimerList is not None:
				for x in simulTimerList:
					if x.setAutoincreaseEnd(entry):
						self.session.nav.RecordTimer.timeChanged(x)
				simulTimerList = self.session.nav.RecordTimer.record(entry)
				if simulTimerList is not None:
					self.session.openWithCallback(self.cbTimerSanityCorrection, TimerSanityConflict, simulTimerList)
			else:
				msg = _("Timer erfolgreich angelegt!")
				MsgBox = self.session.open(MessageBox, msg, MessageBox.TYPE_INFO,timeout = 4)
				MsgBox.setTitle(_("Timer"))
		
	def cbTimerSanityCorrection(self, answer):
		self.cbTimerAdd(answer)
		
	def showEndMarker(self,side):
		if side == "left":
			self["endLeftMarker"].show()
		else:
			self["endRightMarker"].show()
		self.hideEndMarkerTimer.stop()
		self.hideEndMarkerTimer.start(300,True)
	
	def onHideEndMarkerTimer(self):
		self["endLeftMarker"].hide()
		self["endRightMarker"].hide()
		
	def hideCoverBigBefore(self):
		if self["coverBig"].visible:
			self["coverBig"].hide()
			self["coverBigBlur"].hide()
			return True
		else:
			return False
		
	def searchByAirtime(self,sAirtime):
		writeLog("searchByAirtime: " + str(sAirtime))
		sTVSDate = str(datetime.fromtimestamp(sAirtime).strftime("%Y-%m-%d"))
		if self.sTVSCode and sTVSDate:
			#Search AirtimeDate
			self.getTVSpielfilm(self.sTVSCode,sTVSDate)
		
			if len(self.sTVSResultList) > 0:
				#Search AirtimeDate - 1
				if sAirtime < self.sTVSResultList[0]["timestart"]:
					sTVSDate = self.tvsDateAdd(sTVSDate,-1)
					self.getTVSpielfilm(self.sTVSCode,sTVSDate)
				#Search AirtimeDate + 1
				elif sAirtime > self.sTVSResultList[-1]["timeend"]:
					sTVSDate = self.tvsDateAdd(sTVSDate,1) 
					self.getTVSpielfilm(self.sTVSCode,sTVSDate)
		
		res = None
		if len(self.sTVSResultList):
			if sAirtime < self.sTVSResultList[0]["timestart"]:
				res = self.sTVSResultList[0]
				self.showEndMarker("left")
			elif sAirtime >= self.sTVSResultList[-1]["timeend"]:
				res = self.sTVSResultList[-1]
				self.showEndMarker("right")
			else:	
				for entry in self.sTVSResultList:
					if sAirtime >= entry["timestart"] and sAirtime < entry["timeend"]:
						res = entry
						break
					
		if res:
			if self.nextPrevAirtime:
				if sAirtime >= (self.sTVSResultList[0]["timestart"] - 86400) and sAirtime <= (self.sTVSResultList[-1]["timestart"] + 86400):
					self.nextPrevAirtime = sAirtime
				
		self.showEntry(res,False)
			
	def findAirtimeIndex(self,airtime):
		writeLog("findAirtimeIndex: " + str(airtime))
		for i, entry in enumerate(self.sTVSResultList):
			if entry["timestart"] == airtime:
				return i
		return -1
				
	def showEntry(self,entry,coverThread = True):
		if entry:
			writeLog("showEntry: " + str(entry["timestart"]))
			self.currentEntry = entry
			self.currentAirtime = entry["timestart"]
			
			self["eventtitle"].setText(entry["eventtitle"])
			self["infoline"].setText(entry["infoline"])
			self["eventtime"].setText(entry["eventtime"])
			self["eventdate"].setText(entry["eventweekday"] + "  " + entry["eventdate"])
			self["eventtimebig"].setText(entry["eventtime"])
			self["eventdatebig"].setText(entry["eventweekday"] + "  " + entry["eventdate"])
			self["duration"].setText(entry["duration"])
			self["description"].setText(entry["description"])
			self["casting"].setText(entry["casting"])
			
			self["ratingAction"].setPixmapNum(entry["ratingAction"])
			self["ratingSuspense"].setPixmapNum(entry["ratingSuspense"])
			self["ratingHumor"].setPixmapNum(entry["ratingHumor"])
			self["ratingDemanding"].setPixmapNum(entry["ratingDemanding"])
			self["ratingErotic"].setPixmapNum(entry["ratingErotic"])
			
			self["isNew"].setPixmapNum(entry["isNew"])
			self["isTipOfTheDay"].setPixmapNum(entry["isTipOfTheDay"])
			self["isTopTip"].setPixmapNum(entry["isTopTip"])
			
			#ProgressBar
			now = int(time.time())
			if now >= entry["timestart"] and now < entry["timeend"]:
				self["progressBar"].setValue(((now-entry["timestart"])*100)/(entry["timeend"]-entry["timestart"]))
				self["progressBar"].show()
			else:
				self["progressBar"].hide()
				
			#Auto-PrimeTimeMode 
			if self.primeTime >= entry["timestart"] and self.primeTime < entry["timeend"]:
				self.setPrimeTimeMode(self.primeTime)
			
			#Cover
			if entry["imageUrl"]:
				if not self.coverStartup:
					self.showCoverDelayTimer.stop()
					self.showCoverDelayTimer.start(self.coverDelayZap, True)
				elif self.coverDelayStartup > 0:
					self.coverStartup = False
					self.showCoverDelayTimer.stop()
					self.showCoverDelayTimer.start(self.coverDelayStartup, True)
				elif self.coverDelayStartup == 0:
					self.coverStartup = False
					self.onCoverDelayTimer(True)
				
			#Trailer
			if entry["trailerUrl"].endswith("mp4"):
				self["trailer"].show()
			else:
				self["trailer"].hide()
			
		#======== no entry ========================
		else:
			writeLog("no entry found")
			self.currentEntry = None
			self.currentAirtime = 0
			
			self["eventtitle"].setText(_("Keine Daten gefunden  (TVS-Setup prüfen)"))
			self["infoline"].setText("")
			self["eventtime"].setText("")
			self["eventdate"].setText("")
			self["eventtimebig"].setText("")
			self["eventdatebig"].setText("")
			self["duration"].setText("")
			self["description"].setText("")
			self["casting"].setText("")
			
			self["ratingAction"].setPixmapNum(0)
			self["ratingSuspense"].setPixmapNum(0)
			self["ratingHumor"].setPixmapNum(0)
			self["ratingDemanding"].setPixmapNum(0)
			self["ratingErotic"].setPixmapNum(0)
			
			self["isNew"].setPixmapNum(0)
			self["isTipOfTheDay"].setPixmapNum(0)
			self["isTopTip"].setPixmapNum(0)
			
			self["progressBar"].hide()
			
			self["cover"].hide()
			self["trailer"].hide()
			
		self.coverStartup = False
			
	def onCoverDelayTimer(self,coverSync=False):
		if not self.currentEntry: return
		if self.currentEntry["imageUrl"]:
			if not coverSync:
				start_new_thread(self.getCover, (self.currentEntry["imageUrl"],))
			else:
				self.getCover(self.currentEntry["imageUrl"])
		else:
			self["cover"].hide()
			
	def getCover(self,url):
		if self.downloadTVSImage(url, coverPath):
			self.showCover(self["cover"],coverPath)
			if not self.currentEntry:
				self["cover"].hide()
		else:
			self["cover"].hide()
			
	def downloadTVSImage(self,url, imgPath):
		try:
			ir = requests.get(url, stream=True, timeout=4)
			if ir.status_code == 200:
				with open(imgPath, 'wb') as f:
					ir.raw.decode_content = True
					shutil.copyfileobj(ir.raw, f)
					f.close()
				ir = None
				return True
			else:
				return False
		except Exception as ex:
			return False
			
	def showCover(self, pix, imgPath):
		self.picload = ePicLoad()
		if fileExists(imgPath):
			pix.instance.setPixmap(gPixmapPtr())
			scale = AVSwitch().getFramebufferScale()
			size = pix.instance.size()
			self.picload.setPara((size.width(),size.height(),0,0, False, 1, '#000000'))
			if self.picload.startDecode(imgPath, 0, 0, False) == 0:
				ptr = self.picload.getData()
				if ptr != None:
					pix.instance.setPixmap(ptr)
					pix.show()
			del self.picload
	
	def setPrimeTimeMode(self,val):
		self.primeTimeMode = val
		if self.primeTimeMode:
			self["primeTimeMode"].show()
		else:
			self["primeTimeMode"].hide()
			
	def convertSE(self,season,episode):
		se = ""
		if season:
			if len(season) == 1:
				season = "0" + season
			se = "S" + season
		if episode:
			if "/" in episode:
				episodeAct,episodeAll = episode.split("/")
				episodeAct = episodeAct.strip()
				episodeAll = episodeAll.strip()
				if len(episodeAct) == 1: 
					episodeAct = "0" + episodeAct
				if len(episodeAll) == 1: 
					episodeAll  = "0" + episodeAll
				episode = episodeAct + " / " + episodeAll
			else:
				if len(episode) == 1:
					episode = "0" + episode
			episode = "E" + episode
			se = se + " - " + episode
		se = se.strip(" -")
		return se
		
	def getPlayingService(self):
		service = None
		eServiceRef = self.session.nav.getCurrentlyPlayingServiceReference()
		if eServiceRef:
			service = ServiceReference(eServiceRef)
		return service
		
	def getNextPrevService(self, addVal=1):
		service = None
		if self.currentService and self.serviceList:
			currentIndex = -1
			for index, service in enumerate(self.serviceList):
				if str(service) == str(self.currentService):
					currentIndex = index
					break
			if currentIndex >= 0:
				index = currentIndex + addVal
				if index < 0:
					service = self.serviceList[-1]
				elif index > len(self.serviceList)-1:
					service = self.serviceList[0]
				else:
					service = self.serviceList[index]
		return service
		
	def getNextPrevMovieRef(self,addValue):
		newMovieService = None
		if not len(self.movieList):
			self.getMovieList()
		try:
			if self.currentService and len(self.movieList):
				idx = self.movieList.index(self.currentService.ref) + addValue
				if idx < len(self.movieList) and idx >= 0:
					newMovieService = self.movieList[idx]
				elif idx < 0:
					newMovieService = self.movieList[0]
				elif idx >= len(movieList):
					newMovieService = self.movieList[-1]
		except:
			pass
		return 	newMovieService
		
	def getMovieList(self):
		try:
			if self.movieSelection:
				for item in self.movieSelection.list.list:
					if item[0].type != 2:
						self.movieList.append(item[0]) #eServiceRef
		except:
			pass
			
	def updateMovieSelection(self):
		if self.callType == "MovieList" and self.movieSelection and self.currentService:
			if self.currentService.ref in self.movieList:
				try:
					self.movieSelection.list.moveTo(self.currentService.ref)
				except:
					pass
		
	def getServiceList(self):
		services = []
		actBouquetName = ""
		actBouquet = None
		if InfoBar and InfoBar.instance:
			actBouquet = InfoBar.instance.servicelist.getRoot()
		if actBouquet:
			actBouquetName = ServiceReference(actBouquet).getServiceName()
			servicelist = eServiceCenter.getInstance().list(actBouquet)
			if not servicelist is None:
				while True:
					service = servicelist.getNext()
					if not service.valid(): #check end of list
						break
					playable = not (service.flags & (eServiceReference.isMarker|eServiceReference.isDirectory))
					if playable:
						services.append(ServiceReference(service))
		return (services, actBouquetName)
		
	def load_json(self, filename):
		f = open(filename,'r')
		data=f.read().replace('null', '""')
		f.close()
		return eval(data)
		
	def cleanUp(self):
		del self.sTVSResultList
		del self.sTVSSearchedDatesList
		self.movieSelection = None
		self.movieContextMenu = None
		
	def getPrimeTime(self):
		ptHour = config.plugins.TVSViewer.primeTime.value[0]
		ptMin = config.plugins.TVSViewer.primeTime.value[1]
		dt = datetime.fromtimestamp(int(time.time()))
		if (dt.hour*60 + dt.minute) > (ptHour*60 + ptMin + 60):
			dt = dt + timedelta(days=1) # Tageswechsel
		dt = datetime(dt.year, dt.month, dt.day, ptHour, ptMin)
		primeTime = time.mktime(dt.timetuple()) + dt.microsecond/1e6
		return primeTime
		
	def tvsDateAdd(self,tvsDate,daysToAdd):
		try:
			dt = datetime.strptime(tvsDate, "%Y-%m-%d")
			dt = dt + timedelta(days=daysToAdd)
			return str(dt.strftime("%Y-%m-%d"))
		except:
			return ""
			
	def getTVSCode(self,ref):
		tvsCode = ""
		tvsref = {}
		if ref and fileExists(self.tvsRefFile):
			tvsref = self.load_json(self.tvsRefFile)
			tvsCode = str(tvsref.get(ref,"")).upper()
		return tvsCode
			
	def getTVSpielfilm(self,sTVSCode,sTVSDate):
		writeLog("getTVSpielfilm: " + str(sTVSCode) + " | " + str(sTVSDate))
		if sTVSDate in self.sTVSSearchedDatesList:
			return
		try:
			now = int(time.time())
			sRegionalCode = ""
			if "+" in sTVSCode:
				sTVSCode,sRegionalCode = sTVSCode.split("+",1)
				
			url = 'https://live.tvspielfilm.de/static/broadcast/list/' + sTVSCode + '/' + sTVSDate
			results = json.loads(requests.get(url, timeout=4).text)
			if results:
				for event in results:
					if not (event.get("timestart",False) and event.get("timeend",False)):
						continue
					else:
						#timestart/timeend
						timestart = int(event.get("timestart",-1))
						timeend = int(event.get("timeend",-1))
						
						#eventtitle
						eventtitle = str(event.get("title",""))
						
						#Infoline
						genre = str(event.get("genre",""))
						country = str(event.get("country",""))
						year = str(event.get("year",""))
						fsk = str(event.get("fsk",""))
						if fsk:
							fsk = "FSK" + fsk
						category = str(event.get("sart_id",""))
						category = category.replace('U', 'Unterhaltung').replace('SE', 'Serie').replace('SPO', 'Sport').replace('SP', 'Spielfilm').replace('KIN', 'Kinder').replace('RE', 'Reportage').replace('AND', '')
						infoline = " - ".join(x.strip() for x in (genre,country,year,fsk) if x.strip())
						if infoline and category:
							infoline = infoline + "   (" + category + ")"
						
						#eventtime/eventdate/eventweekday
						startTime = datetime.fromtimestamp(timestart)
						startStr = startTime.strftime("%H:%M")
						endStr = datetime.fromtimestamp(timeend).strftime("%H:%M")
						eventtime = startStr + " - " + endStr
						eventdate = startTime.strftime("%d.%m.%Y")
						eventweekday = ("Mo.","Di.","Mi.","Do.","Fr.","Sa.","So.")[startTime.weekday()]
						
						#duration
						duration = str(int((timeend-timestart)/60)) + " min"
						
						#description
						desclist = ""
						se = self.convertSE(str(event.get("seasonNumber","")), str(event.get("episodeNumber","")))
						episodetitle = str(event.get("episodeTitle",""))
						if se and episodetitle:
							desclist = desclist	+ se + ": " + episodetitle + "\n\n"
						elif se:
							desclist = desclist + se + "\n\n"
						elif episodetitle:
							desclist = desclist + episodetitle + "\n\n"
						
						preview = str(event.get("preview",""))
						if preview:
							desclist = desclist + preview + "\n\n"
							
						repeatHint = str(event.get("repeatHint",""))
						if repeatHint:
							desclist = desclist + repeatHint + "\n\n"
						
						desc = str(event.get("text",""))
						if desc:
							desclist = desclist + desc
						description = desclist.rstrip("\n")
						
						#casting
						castinglist = ""
						conclusion = str(event.get("conclusion",""))
						if conclusion:
							castinglist = castinglist +  "Kritik: " + conclusion + "\n\n"
						director = str(event.get("director",""))
						if director:
							castinglist = castinglist + "Regie: " + director + "\n\n"
						actors = event.get("actors",[])
						if len(actors) > 0:
							for item in actors:
								for k, v in item.items():
									castinglist = castinglist + str(k) + " (" + str(v) +")\n"
						casting = castinglist.rstrip("\n")
						
						#ratings
						ratingAction = int(event.get("ratingAction",0))
						ratingDemanding = int(event.get("ratingDemanding",0))
						ratingErotic = int(event.get("ratingErotic",0))
						ratingHumor = int(event.get("ratingHumor",0))
						ratingSuspense = int(event.get("ratingSuspense",0))
						
						#Tipps
						isNew = int(event.get("isNew",False) == True)
						isTipOfTheDay = int(event.get("isTipOfTheDay",False) == True)
						isTopTip = int(event.get("isTopTip",False) == True)
						
						#image
						imageUrl = ""
						if "images" in event:
							imageUrl = event["images"][0]["size4"]
							
						#trailerUrl
						trailerUrl = ""
						if "videos" in event:
							trailerUrl = str(event["videos"][0]["video"][0]["url"])
						
							
						#newEntry
						stillInList = False
						for entry in self.sTVSResultList:
							if entry["timestart"] == timestart:
								stillInList = True
								
						#OutofRegion
						outOfRegion = False
						if sRegionalCode and eventtitle:
							if sRegionalCode == "BW" and (eventtitle.startswith("RP:") or eventtitle.startswith("SR:")):
								outOfRegion = True
							elif sRegionalCode == "RP" and (eventtitle.startswith("BW:") or eventtitle.startswith("SR:")):
								outOfRegion = True
							elif sRegionalCode == "SR" and (eventtitle.startswith("BW:") or eventtitle.startswith("RP:")):
								outOfRegion = True
								
						#tooOld
						tooOld = False
						if not self.sTVSAgeFilterOff and timestart < (now-10800) and not (timeend >= now):
							tooOld = True
						
						if not (stillInList or tooOld or outOfRegion):
							newEntry = {
								"tvsDate" : sTVSDate,
								"timestart" : timestart,
								"timeend" : timeend,
								"eventtitle" : eventtitle,
								"infoline" : infoline,
								"eventtime" : eventtime,
								"eventdate" : eventdate,
								"eventweekday" : eventweekday,
								"duration" : duration,
								"episodetitle" : episodetitle,
								"preview" : preview,
								"description" : description,
								"casting" : casting,
								"ratingAction" : ratingAction,
								"ratingDemanding" : ratingDemanding,
								"ratingErotic" : ratingErotic,
								"ratingHumor" : ratingHumor,
								"ratingSuspense" : ratingSuspense,
								"isNew" : isNew,
								"isTipOfTheDay" : isTipOfTheDay,
								"isTopTip" : isTopTip,
								"imageUrl" : imageUrl,
								"trailerUrl" : trailerUrl,
							}
							self.sTVSResultList.append(newEntry)
				self.sTVSSearchedDatesList.append(sTVSDate)
				self.sTVSResultList.sort(key=lambda x: x["timestart"])
				writeLog("sTVSResultList-Count: " + str(len(self.sTVSResultList)))
				#writeLog(self.sTVSResultList)
		except Exception as ex:
			writeLog("Error-getTVSpielfilm: " + str(ex))

#######################################################
class TVSViewer_TimerEPG(Screen):
	def __init__(self, session, entry, service):
		Screen.__init__(self, session)
		
		self.session = session
		self.title = "EPG - Zuordnung"
		self.skinName = "TVSViewer_TimerEPG"
		
		self.entry = entry
		self.service = service
		
		self.epgEvent = None
		
		self["title"] = StaticText(self.title)
		self["background"] = Label()
		self["chInfo"] = Label()
		self["tvsTime"] = Label()
		self["tvsEventTitle"] = Label()
		self["epgTime"] = Label()
		self["epgEventTitle"] = Label()
		
		self["key_red"] = Label(_("Abbrechen"))
		self["key_green"] = Label()
		
		self["myActionMap"] = ActionMap(["TVSViewerActions"],
		{
			"key_red": self.keyRed,
			"key_green": self.keyGreen,
			"key_ok": self.keyGreen,
			"key_cancel": self.keyRed,
		},-1)
		
		self.onLayoutFinish.append(self.startup)
		
	def startup(self):
		#background
		bkColor = config.plugins.TVSViewer.skin.value
		self["background"].instance.setBackgroundColor(parseColor(bkColor))
		
		#TVS-Data
		tsStart = self.entry["timestart"]
		tsEnd = self.entry["timeend"]
		tvsTime = str(datetime.fromtimestamp(tsStart).strftime("%H:%M")) + " - " + str(datetime.fromtimestamp(tsEnd).strftime("%H:%M"))
		self["chInfo"].setText(self.entry["eventweekday"] + "  " +  self.entry["eventdate"] + "  -  " + str(self.service.getServiceName()))
		self["tvsTime"].setText(tvsTime)
		self["tvsEventTitle"].setText(self.entry["eventtitle"])
		
		try:
			#EPG-Data
			epgcache = eEPGCache.getInstance()
			sAirtime = tsStart + ((tsEnd - tsStart) / 4)
			self.epgEvent = epgcache.lookupEventTime(self.service.ref, sAirtime)
			if self.epgEvent:
				epgTime = str(datetime.fromtimestamp(self.epgEvent.getBeginTime()).strftime("%H:%M")) + " - " + str(datetime.fromtimestamp(self.epgEvent.getBeginTime() + self.epgEvent.getDuration()).strftime("%H:%M"))
				self["epgTime"].setText(epgTime)
				self["epgEventTitle"].setText(self.epgEvent.getEventName())
				self["key_green"].setText(_("Übernehmen"))
		except:
			pass
		
		if not self.epgEvent:
			self["epgTime"].setText("---")
			self["epgEventTitle"].setText(_("Keine EPG-Zuordnung gefunden"))
	
	def keyRed(self):
		self.close(None)
		
	def keyGreen(self):
		if self.epgEvent and self.service:
			timer = RecordTimerEntry(self.service, *parseEvent(self.epgEvent))
			timer.repeated = 0
			timer.tags = ['TVS-Viewer']
			self.close(timer)

#######################################################
class TVSViewer_MoviePlayer(MoviePlayer):
	def __init__(self, session, service):
		MoviePlayer.__init__(self, session, service)
		self.skinName = "MoviePlayer"

	def leavePlayer(self):
		self.close()
		
	def doEofInternal(self, playing):
		if self.execing and playing:
			self.close()
		
	def showMovies(self): #key up/down smartseek off
		pass
		
	def seekFwd(self):
		self.smartSeek_Key_Right()
		
	def seekBack(self):
		self.smartSeek_Key_Left()
		
	def smartSeek_Key_Up(self):
		pass

	def smartSeek_Key_Down(self):
		pass
		
	def smartSeek_Key_Left(self):
		if self.seekstate != self.SEEK_STATE_PLAY:
			self.unPauseService()
		time = -10
		self.setSeekStateText(time)
		self.doSeekRelative(time * 90000)
		
	def smartSeek_Key_Right(self):
		if self.seekstate != self.SEEK_STATE_PLAY:
			self.unPauseService()
		time = 10
		self.setSeekStateText(time)
		self.doSeekRelative(time * 90000)