# - *- coding: utf- 8 - *-

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.Standby import TryQuitMainloop

from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigYesNo, ConfigInteger, ConfigDescription, ConfigClock, fileExists
from Components.ConfigList import ConfigList, ConfigListScreen
from enigma import getDesktop

import time

#============ Version =============
version = "V1.3"

#============ check installed Plugins =============
isTMDbInstalled = fileExists("/usr/lib/enigma2/python/Plugins/Extensions/tmdb/plugin.pyo")
isAELInstalled = fileExists("/usr/lib/enigma2/python/Plugins/Extensions/AdvancedEventLibrary/AdvancedEventLibrarySystem.pyo") and fileExists("/usr/lib/enigma2/python/Plugins/Extensions/AdvancedEventLibrary/tvsreflist.data")

#============ Pathes =============
pluginPath = "/usr/lib/enigma2/python/Plugins/Extensions/TVSpielfilmViewer/"
skinPath = pluginPath + "skins/FHD/"
if getDesktop(0).size().width() < 1920:
	skinPath = pluginPath + "skins/HD/"
coverPath = pluginPath + "images/Cover.jpg"
tvsRefFileIntern = pluginPath + "tvsreflist.data"
tvsRefFileAEL = "/usr/lib/enigma2/python/Plugins/Extensions/AdvancedEventLibrary/tvsreflist.data"

#============ Log =============
logging = False
logFile = "/tmp/TVS-Viewer.log"
def enableLogging():
	global logging
	logging = True
def writeLog(logText):
	if not logging: return
	t = time.localtime()
	logTime = time.strftime("%d.%m.%Y %H:%M:%S: ",t)
	log = open(logFile,"a")
	log.write(str(logTime) + str(logText) + "\n")
	log.close()

#============ config =============
config.plugins.TVSViewer = ConfigSubsection()
config.plugins.TVSViewer.skin = ConfigSelection(default="#30000000", choices=[("#30000000",_("Transp. Black")),("#30090930",_("Transp. BlueViolet")),("#000000",_("Black"))])
config.plugins.TVSViewer.coverDelayStartup = ConfigInteger(default = 50, limits = (0, 999))
config.plugins.TVSViewer.coverDelayZap = ConfigInteger(default = 150, limits = (0, 999))
config.plugins.TVSViewer.primeTime = ConfigClock(default = 69300) # 20:15
if isAELInstalled:
	config.plugins.TVSViewer.tvsSetup = ConfigSelection(default="TVS-Viewer", choices=[("TVS-Viewer",_("TVS-Viewer")), ("AEL",_("AEL"))])
else:
	config.plugins.TVSViewer.tvsSetup = ConfigSelection(default="TVS-Viewer", choices=[("TVS-Viewer",_("TVS-Viewer"))])
config.plugins.TVSViewer.asSIB = ConfigYesNo(default=False)
config.plugins.TVSViewer.moviePlayerKeyGreen = ConfigYesNo(default=False)
config.plugins.TVSViewer.logging = ConfigYesNo(default=False)


#============ TVSViewerConfigScreen =============
class TVSViewerConfigScreen(Screen, ConfigListScreen):
	def __init__(self, session):
		Screen.__init__(self, session)
		self.skinName = [ "TVSViewerConfigScreen", "Setup" ]
		self.setTitle(_("TV-Spielfilm-Viewer - Einstellungen") + " " + version)
		
		self.list = []
		self.buildConfigList()
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changedEntry)
		
		self["config"].onSelectionChanged.append(self.updateDesc)
		
		self["actions"] = ActionMap(["ColorActions", "SetupActions"],
			{
				"cancel": self.keyCancel,
				"save": self.keyOK,
				"red": self.keyCancel,
				"green": self.keyGreen,
			}, -2)

		self["key_green"] = StaticText(_("Save"))
		self["key_red"] = StaticText(_("Cancel"))

	def buildConfigList(self):
		self.list = []
		
		self.list.append(getConfigListEntry(_("Skin-Auswahl"), config.plugins.TVSViewer.skin, ""))
		
		self.list.append(getConfigListEntry(_("Cover-Anzeige verzögern (Plugin-Start) [ms]"), config.plugins.TVSViewer.coverDelayStartup, _("Wert erhöhen, wenn durch die  Ladezeiten von Covern der Plugin-Start verzögert wird. Achtung: Bei Wert genau 0 wird das Plugin erst angezeigt, wenn das Cover geladen wurde [Default: 50ms]")))
		
		self.list.append(getConfigListEntry(_("Cover-Anzeige verzögern (Sendungswechsel) [ms]"), config.plugins.TVSViewer.coverDelayZap, _("Wert erhöhen, wenn durch die  Ladezeiten von Covern die Bedienung beeinträchtigt wird [Default: 150ms]")))
		
		self.list.append(getConfigListEntry(_("PrimeTime"), config.plugins.TVSViewer.primeTime, _("Hinweis: Wird das PrimeTime-Symbol angezeigt, wird auch bei Kanalwechsel im Plugin, nach der PrimeTime auf dem neuen Sender gesucht")))
		
		self.list.append(getConfigListEntry(_("TVS-Setup-Auswahl"), config.plugins.TVSViewer.tvsSetup, _("TVS-Setup von TVS-Viewer oder von AEL (Avanced-Event-Library) verwenden")))
		
		self.list.append(getConfigListEntry(_("TVS-Viewer als SIB benutzen"), config.plugins.TVSViewer.asSIB, _("Das Plugin anstelle der Second-Infobar anzeigen")))
		
		self.list.append(getConfigListEntry(_("TVS-Viewer mit Taste-Grün im MoviePlayer öffnen"), config.plugins.TVSViewer.moviePlayerKeyGreen, _("Taste Grün -> TVS-Viewer öffnen, Taste Grün-Long -> Wiedergabemodus ändern\n(Die Taste Grün im MoviePlayer kann mit MQB nicht überschrieben werden)")))
		
		self.list.append(getConfigListEntry(_("Schreibe Logdatei"), config.plugins.TVSViewer.logging, _("Logdatei: /tmp/TVS-Viewer.log")))
		
	def changedEntry(self):
		self.buildConfigList()
		self["config"].setList(self.list)
	
	def updateDesc(self):
		cur = self["config"].getCurrent()
		if cur and len(cur)>2:
			self["configdesc"].text = cur[2]
	
	def keyOK(self):
		return
		#self.keyGreen()

	def keyGreen(self):
		for x in self["config"].list:
			x[1].save()
		#Restart GUI
		msg = self.session.openWithCallback(self.restartGUI,MessageBox,_("Restart necessary, restart GUI now?"), MessageBox.TYPE_YESNO)
		msg.setTitle(_("Message"))
		
	def restartGUI(self,answer):		
		if answer:
			self.session.open(TryQuitMainloop, 3)
		else:
			self.close()
			
			
		