# - *- coding: utf- 8 - *-
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Components.Label import Label
from Components.ConfigList import ConfigListScreen
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigYesNo, ConfigInteger, ConfigDescription, ConfigClock, ConfigText, fileExists
from Components.Sources.StaticText import StaticText
from Components.Sources.List import List
from Components.ActionMap import ActionMap
from skin import parseColor

from .TVSViewerConfig import TVSViewerConfigScreen, pluginPath, skinPath, tvsRefFileIntern, writeLog

import json
import requests
import skin
import string
import os

#######################################################
class TVSViewer_TVSSetup(Screen, ConfigListScreen):
	skin.loadSkin(skinPath + "TVSViewer_TVSSetup.xml")
	def __init__(self, session,serviceList,curRef,curName):
		Screen.__init__(self, session)
		self.session = session
		self.skinName = "TVSViewer_TVSSetup"
		self.title = "TVS-Setup"
		
		self.serviceList = []
		self.bouquetName = ""
		self.curRef = curRef
		self.curName = curName
		
		if serviceList:
			self.serviceList = serviceList[0]
			self.bouquetName = serviceList[1]
		
		self.tvsRefList = self.getTVSRefList()
		self.tvsCodeList = []
		
		self.startupConfiglist = []
		
		self["title"] = StaticText(self.title)
		self["background"] = Label()
		self["bouquet"] = Label(self.bouquetName)
		self["info"] = Label()

		self["key_red"] = Label(_("Beenden"))
		self["key_green"] = Label(_("Speichern"))
		self["key_yellow"] = Label("Löschen")
		self["key_blue"] = Label(_("Auto-Zuordnung"))
		
		self.configlist = []
		self.buildConfigList()
		ConfigListScreen.__init__(self, self.configlist, session = self.session, on_change = self.changedEntry)

		self["myActionMap"] = ActionMap(["TVSViewerActions"],
		{
			"key_red": self.keyRed,
			"key_green": self.keyGreen,
			"key_yellow": self.keyYellow,
			"key_blue": self.keyBlue,
			"key_ok": self.keyOK,
			"key_cancel": self.keyRed,
			"key_channel_up": self.keyChannelUp,
			"key_channel_down": self.keyChannelDown,
		}, -1)
		
		self["config"].onSelectionChanged.append(self.updateInfo)
		
		self.onLayoutFinish.append(self.startup)

	def startup(self):
		#background
		bkColor = config.plugins.TVSViewer.skin.value
		self["background"].instance.setBackgroundColor(parseColor(bkColor))
		
		#curRef anwaehlen
		if self.curRef:
			for index,confEntry in enumerate(self["config"].list):
				if str(confEntry[2]) == self.curRef:
					self["config"].setCurrentIndex(index)
					break
		
	def keyOK(self):
		if not len(self.tvsCodeList):
			self.tvsCodeList = self.getTVSCodeList()
		
		if len(self.tvsCodeList) and len(self["config"].list):
			preSelTVSCode = None
			cur = self["config"].getCurrent()
			if cur:
				preSelTVSCode = cur[1].value
			self.session.openWithCallback(self.cbCodeList, TVSViewer_TVSSetup_CodeList, self.tvsCodeList, preSelTVSCode)
			
	def cbCodeList(self, answer = None):
		cur = self["config"].getCurrent()
		if cur and answer:
			if answer == "---":
				cur[1].value = ""
			else:
				cur[1].value = answer
				
	def keyRed(self):
		#Close
		if self.checkChanged():
			msgBox = self.session.openWithCallback(self.cbClose,MessageBox,_("Änderungen verwerfen und TVS-Setup beenden?"), MessageBox.TYPE_YESNO, default = False)
			msgBox.setTitle(_("Änderungen verwerfen"))
		else:
			self.close()
			
	def cbClose(self,answer):
		#Close
		if answer:
			self.close()
		
	def keyGreen(self):
		#Save
		if len(self["config"].list):
			msgBox = self.session.openWithCallback(self.cbSave,MessageBox,_("Einstellungen speichern und TVS-Setup beenden?"), MessageBox.TYPE_YESNO)
			msgBox.setTitle(_("Einstellungen speichern"))
		
	def cbSave(self,answer):
		#Save
		if answer:
			for confEntry in self["config"].list:
				ref = confEntry[2]
				tvsCode = confEntry[1].value
				if ref:
					if ref in self.tvsRefList:
						self.tvsRefList[ref] = tvsCode
					else:
						self.tvsRefList.update({ref:tvsCode})
			self.save_json(self.tvsRefList,tvsRefFileIntern)
			self.close()
			
	def keyYellow(self):
		#Delete
		msgBox = self.session.openWithCallback(self.cbDelete,MessageBox,_("TVS-Setup für alle Bouquets komplett löschen?\n\nDas TVS-Setup wird anschliessend beendet und muss neu aufgerufen werden"), MessageBox.TYPE_YESNO)
		msgBox.setTitle(_("TVS-Setup löschen"))
		
	def cbDelete(self,answer):
		#Delete
		if answer:
			try:
				if fileExists(tvsRefFileIntern):
					os.remove(tvsRefFileIntern)
			except:
				pass
			self.close()
		
	def keyBlue(self):
		#Auto-Zuordnung
		if len(self.configlist):
			msg = _("Versuchen TVS-Codes automatisch zuzuordnen?\n\nBereits eingetragene Codes werden nicht überschrieben. Nicht oder fehlerhaft zugeordnete Einträge, können über die OK-Taste zugewiesen/geändert werden")
			msgBox = self.session.openWithCallback(self.cbMsgBoxAutoTVSCode, MessageBox, msg, MessageBox.TYPE_YESNO)
			msgBox.setTitle(_("Auto-TVS-Code"))
		
	def cbMsgBoxAutoTVSCode(self, answer = None):
		#Auto-Zuordnung
		if answer:
			writeLog("Auto-Zuordnung: Start")
			if not len(self.tvsCodeList):
				self.tvsCodeList = self.getTVSCodeList()
			if len(self.tvsCodeList):
				for conf in self.configlist:
					found = False
					if conf[0] and not conf[1].value:
						writeLog("Auto-Zuordnung-Search for Name: " + str(conf[0]))
						#Create SearchNames
						searchNames=[]
						a = conf[0]
						searchNames.append(a)
						b = conf[0].replace(" HD","").replace("_","").replace("-"," ")
						if b and not b in searchNames:
							searchNames.append(b)
						if b:
							bs = b.split()
							if bs and len(bs):
								c = bs[0]
								if c and not c in searchNames:
									searchNames.append(c)
						
						#Search
						for searchName in searchNames:
							writeLog("Auto-Zuordnung-SearchName: " + str(searchName))
							for codeEntry in self.tvsCodeList:
								if searchName.upper() == codeEntry.get("chName","").upper():
									conf[1].value = codeEntry.get("tvsCode","")
									found = True
									writeLog("Auto-Zuordnung-Found: SearchName: " + str(searchName) + "TVS-Code: " + str(conf[1].value))
									break
								elif searchName.upper() == codeEntry.get("chNameShort").upper():
									conf[1].value = codeEntry.get("tvsCode","")
									found = True
									writeLog("Auto-Zuordnung-Found: SearchName: " + str(searchName) + "TVS-Code: " + str(conf[1].value))
									break
								elif searchName.upper() == codeEntry.get("tvsCode").upper():
									conf[1].value = codeEntry.get("tvsCode","")
									found = True
									writeLog("Auto-Zuordnung-Found: SearchName: " + str(searchName) + "TVS-Code: " + str(conf[1].value))
									break
							if found:
								break
							
	def keyChannelUp(self):
		self["config"].pageUp()
		
	def keyChannelDown(self):
		self["config"].pageDown()
					
	def getTVSRefList(self):
		tvsRefList = {}
		try:
			if fileExists(tvsRefFileIntern):
				tvsRefList = self.load_json(tvsRefFileIntern)
		except:
			pass
		return tvsRefList
		
	def getTVSCodeList(self):
		tvsCodeList = []
		errorMsg = ""
		try:
			url = 'https://live.tvspielfilm.de/static/content/channel-list/livetv'
			results = json.loads(requests.get(url, timeout=4).text)
			if results:
				for item in results:
					tvsCode = str(item.get("id",""))
					chName = str(item.get("name",""))
					chNameShort = str(item.get("name_short",""))
					
					if tvsCode and tvsCode != "SWR":
						tvsCodeList.append({"tvsCode":tvsCode, "chName":chName, "chNameShort":chNameShort})
					
				#Additional Channels
				tvsCodeList.append({"tvsCode":"MFTV1", "chName":"Fußball.TV1", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"DOKUS", "chName":"Dokusat", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SERVUSA", "chName":"ServusTV Österreich", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"VH1", "chName":"MTV 80s", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"RTLPL", "chName":"RTLup", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"EOTV", "chName":"More than Sports", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"BERG", "chName":"Bergblick", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"HSEEXTRA", "chName":"HSE Extra", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"TNT-F", "chName":"Warner TV Film", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"TNT-C", "chName":"Warner TV Comedy", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"TNT-S", "chName":"Warner TV Serie", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYCH", "chName":"Sky Cinema Highlights", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"CIN", "chName":"Sky Cinema Premiere", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKY-A", "chName":"Sky Cinema Action", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKY-F", "chName":"Sky Cinema Family", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYSPL", "chName":"Sky Sport Premier League", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYSG", "chName":"Sky Sport Golf", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYST", "chName":"Sky Sport Tennis", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYSTE", "chName":"Sky Sport Top Event", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYSM", "chName":"Sky Sport Mix", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"HISHD", "chName":"HISTORY Channel", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYRP", "chName":"Sky Replay", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYSH", "chName":"Sky Showcase", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SKYSH", "chName":"Curiosity Channel", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SUPER", "chName":"RTL Super", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"DF1", "chName":"DF1", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SWR+SR", "chName":"SR", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SWR+BW", "chName":"SWR BW", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"SWR+RP", "chName":"SWR RP", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"RTL2", "chName":"RTLZWEI", "chNameShort":"---"})
				tvsCodeList.append({"tvsCode":"PULS4", "chName":"PULS 4 Austria", "chNameShort":"---"})
				
			else:
				errorMsg = _("Codeliste konnte von TV-Spielfilm nicht geladen werden")
		except Exception as ex:
			errorMsg = _("Codeliste konnte von TV-Spielfilm nicht geladen werden") + " - " + str(ex)
			
		if errorMsg:
			msgBox = self.session.open(MessageBox, errorMsg, MessageBox.TYPE_ERROR,timeout = 20)
			msgBox.setTitle(_("Fehler"))
			
		return tvsCodeList

	def buildConfigList(self):
		#Live
		if len(self.serviceList):
			for service in self.serviceList:
				chRef = str(service)
				if chRef and not chRef.startswith("1:0:0:0:"): 
					#remove non-printable
					chName = str(filter(lambda x: x in string.printable,service.getServiceName()))
					tvsCode = self.tvsRefList.get(chRef,"")
					self.configlist.append(getConfigListEntry(chName, ConfigText(default = tvsCode),chRef))
					self.startupConfiglist.append(tvsCode)
		
		#EPG/MovieList
		elif self.curRef and not self.curRef.startswith("1:0:0:0:") and self.curName:
			self.curName = str(filter(lambda x: x in string.printable,self.curName))
			if self.curName:
				tvsCode = self.tvsRefList.get(self.curRef,"")
				self.configlist.append(getConfigListEntry(self.curName, ConfigText(default = tvsCode),self.curRef))
				self.startupConfiglist.append(tvsCode)
				
		if not len(self.configlist):
			self["info"].text = _("keine gültige Kanalreferenz gefunden")
				
			
	def changedEntry(self):
		pass
		
	def updateInfo(self):
		self["info"].text = ""
		cur = self["config"].getCurrent()
		if cur and len(cur)>2:
			self["info"].text = cur[2] + "\n"
		self["info"].text = self["info"].text + _("OK: Codeliste, TXT: man. Eingabe oder Auto-Zuordnung")
			
	def checkChanged(self):
		if len(self.configlist):
			if len(self.configlist) == len(self.startupConfiglist):
				for (index, confEntry) in enumerate(self.configlist):
					if self.startupConfiglist[index] != confEntry[1].value:
						return True
			else:
				return True
		return False
	

	def load_json(self, filename):
		f = open(filename,'r')
		data=f.read().replace('null', '""')
		f.close()
		return eval(data)

	def save_json(self, data, filename):
		f = open(filename, 'w')
		f.write(json.dumps(data, indent=4, sort_keys=False))
		f.close()
		
		
#######################################################		
class TVSViewer_TVSSetup_CodeList(Screen):
	def __init__(self, session,tvsCodeList=None,preSelTVSCode=None):
		Screen.__init__(self, session)
		self.session = session
		self.skinName = "TVSViewer_TVSSetup_CodeList"
		self.title = "TVS - Codeliste"
		
		self.tvsCodeList = tvsCodeList
		self.preSelTVSCode = preSelTVSCode
		
		self["title"] = StaticText(self.title)
		self["background"] = Label()
		
		self["list"] = List([])
		
		self["key_red"] = Label(_("Abbrechen"))
		self["key_green"] = Label(_("Übernehmen"))
		
		self["myActionMap"] = ActionMap(["TVSViewerActions"],
		{
			"key_red": self.keyRed,
			"key_green": self.keyGreen,
			"key_ok": self.keyGreen,
			"key_cancel": self.keyRed,
			"key_channel_up": self.keyChannelUp,
			"key_channel_down": self.keyChannelDown,
		},-1)
		
		self.onLayoutFinish.append(self.startup)

	def startup(self):
		#background
		bkColor = config.plugins.TVSViewer.skin.value
		self["background"].instance.setBackgroundColor(parseColor(bkColor))
		
		
		#codeList
		codeList = []
		for item in self.tvsCodeList:
			chName = item.get("chName","")
			chNameShort = item.get("chNameShort","")
			tvsCode = item.get("tvsCode","")
			
			codeList.append((chName,chNameShort,tvsCode))
		codeList.sort(key = lambda row: row[0].upper())
		codeList.insert(0,("-- unbenutzt ---","","---"))
		
		self["list"].list = codeList
		
		#preSelection
		if self.preSelTVSCode:
			for (index, codeEntry) in enumerate(self["list"].list):
				if codeEntry[2] == self.preSelTVSCode:
					self["list"].setCurrentIndex(index)
	
	def keyRed(self):
		self.close(None)
		
	def keyGreen(self):
		selTVSCode = self["list"].current[2]
		self.close(selTVSCode)
		
	def keyChannelUp(self):
		self["list"].pageUp()
		
	def keyChannelDown(self):
		self["list"].pageDown()
		
		
	
		
		